#include <algorithm>
#include <cmath>
#include <ctime>
#include <iostream>
#include <vector>
#include <boost/lambda/lambda.hpp>
#include <boost/lambda/bind.hpp>
#include <boost/function.hpp>
#include <boost/lambda/if.hpp>
#include <boost/lambda/construct.hpp>
#include <boost/typeof/typeof.hpp>

using namespace boost;
using namespace boost::lambda;
using namespace std;


template<class T>
void print_elmt(const vector<T>& v) {
  cout << "V: ";
  for_each(v.begin(), v.end(), cout << _1 << constant(' '));
  cout << endl;
}

template<class T, template<class U, class V> class Cont>
Cont<T,allocator<T> > filter(const Cont<T,allocator<T> >& t,  function<bool (T)> f) {
  //  Cont<T, allocator<T> > new_c;
  BOOST_TYPEOF(t) new_c;
  for(BOOST_AUTO(i,t.begin()); i!=t.end(); ++i)
    if (f(*i))
      new_c.push_back(*i);
  return new_c;
}

template<class R, class T, template<class U, class V> class Cont>
Cont<R,allocator<R> > build(const Cont<T,allocator<T> >& t1, const Cont<T,allocator<T> >& t2,  function<R (T,T)> f) {
  //  Cont<T, allocator<T> > new_c;
  Cont<R, allocator<R> > new_c;
  BOOST_AUTO(i1, t1.begin());
  BOOST_AUTO(i2, t2.begin());
  for(; i1!=t1.end() && i1!= t2.end(); ++i1,++i2)
    new_c.push_back(f(*i1,*i2));
  return new_c;
}

template<class R, class T, template<class U, class V> class Cont>
Cont<R,allocator<R> > build(const Cont<T,allocator<T> >& t1, const Cont<T,allocator<T> >& t2,  function<bool (T,T)> pred,function<R (T,T)> f) {
  //  Cont<T, allocator<T> > new_c;
  Cont<R, allocator<R> > new_c;
  BOOST_AUTO(i1, t1.begin());
  BOOST_AUTO(i2, t2.begin());
  for(; i1!=t1.end() && i1!= t2.end(); ++i1,++i2) {
    if (pred(*i1, *i2))
      new_c.push_back(f(*i1,*i2));
  }
  return new_c;
}

inline
double my_rand() {
  return static_cast<double>(rand())/RAND_MAX;
}

int
main() {
  
  srand(static_cast<unsigned>(time(NULL)));
  const int Size=10;
 
  vector<double> vd;
  vector<double> vd2;

  for (unsigned i=0; i < Size; ++i)
    vd.push_back(my_rand());
      
  // STL + Boost, no pure STL !!!
  print_elmt(vd);
  
  
  print_elmt(filter<double>(filter<double>(vd,_1<.8),_1>.5));

  for (unsigned i=0; i < Size; ++i) {
    vd.push_back(static_cast<double>(rand())/RAND_MAX);
    vd2.push_back(static_cast<double>(rand())/RAND_MAX);
  }
  print_elmt(vd);
  print_elmt(vd2);
  vector<bool> v=build<bool,double>(vd,vd2, _1<_2);
  for (unsigned i=0; i < v.size(); ++i)
    cout << v[i] << " ";
  cout <<endl;
  
  print_elmt(build<double,double>(vd,vd2, _1<_2, _1-_2));
  
  
  return 0;
}
