#include <iostream>
#include <boost/function.hpp>
#include <boost/bind.hpp>
#include <boost/unordered_map.hpp>
#include <vector>
#include <boost/array.hpp>

using namespace std;
using namespace boost;

class Sprite {
  static int ID;
  int x,y;
  int id;
public:
  Sprite(int _x=0, int _y=0): x(_x), y(_y), id(++ID) {}
  void up() { ++x;}
  void down() { --x;}
  void right() { ++y;}
  void left() { --y;}
  void render() { cout << "ID: " << id << "\n>>X: " << x << ", Y: " << y << endl; }
  void move(int dir) {
    switch(dir) {
    case 0:
      up();
      break;
    case 1:
      down();
      break;
    case 2:
      left();
      break;
    case 3:
      right();
      break;
    default:
      break;
    }
  }
};

int Sprite::ID=0;

struct Action {
  typedef function<void()> action;

  void register_action(char k, action a) { 
    acts[k]=a;
  }

  bool key(char k) {
    if (k=='\n') 
      return false;
    acts[k]();
    return true;
  }
  
  unordered_map<char, function<void()> > acts;
};

int bounded_random(int max) {
  double rnd= static_cast<double>(rand())/RAND_MAX;
  return static_cast<int>(rnd*max);
}


int
main() {
  
  // init part
  const unsigned max_sprite=4;
  array<Sprite,2> players;
  Action Aplayer1;
  Action Aplayer2;

  Aplayer1.register_action('w',bind(&Sprite::up, &players[0]));
  Aplayer1.register_action('s',bind(&Sprite::down, &players[0]));
  Aplayer1.register_action('d',bind(&Sprite::right, &players[0]));
  Aplayer1.register_action('a',bind(&Sprite::left, &players[0]));
  Aplayer1.register_action('q',bind(exit,0));

  Aplayer2.register_action('i',bind(&Sprite::up, &players[1]));
  Aplayer2.register_action('k',bind(&Sprite::down, &players[1]));
  Aplayer2.register_action('l',bind(&Sprite::right, &players[1]));
  Aplayer2.register_action('j',bind(&Sprite::left, &players[1]));
  Aplayer1.register_action('q',bind(exit,0));

  vector<Action*> actions;
  actions.push_back(&Aplayer1);
  actions.push_back(&Aplayer2);
  
  vector<Sprite> sprites;
  for (unsigned i=0; i < max_sprite; ++i) 
    sprites.push_back(Sprite(bounded_random(256), bounded_random(256)));
  
  
  
  // "motor part"
  while (1) {
    
    // get player position
    while (!Aplayer1.key(getchar()));
    while(!(Aplayer2.key(getchar())));

    // bind : one call for all the loop :(
    for_each(sprites.begin(), sprites.end(),bind(&Sprite::move,_1,bounded_random(5)));
    
    for_each(players.begin(), players.end(), mem_fn(&Sprite::render));
    for_each(sprites.begin(), sprites.end(),mem_fn(&Sprite::render));
    
  }
  
  return 0;
}

