#include <iostream>
#include <boost/any.hpp>
#include <vector>
#include <list>
#include <typeinfo>
#include <boost/unordered_map.hpp>
#include <string>
#include <cassert>
using namespace std;
using namespace boost;

// handle should be more opaque

class PropertyManager {
  unordered_map<string, any> _property_pool;
public:
  template<typename Property>
  class Handle {
    typedef Property type;
    type _handle;
    string _name;
    Handle(const type& u, const string& name): _handle(u), _name(name) {}
    Handle() {}

    friend class PropertyManager;
  public:
    template <class T>
    type build_type(const T& t){return type(t);}
  };
  
  template<class Prop>
  Handle<Prop> request_property(const string& name, const Prop& p) {
    _property_pool[name]=p;
    return Handle<Prop>(p, name);
  }
  
  template<class Prop>
  void set_property(const Handle<Prop>& h, const Prop& value) {
    any_cast<Prop&>(_property_pool[h._name])=value;
  }

  template<class Prop>
  Prop& get_property(const Handle<Prop>& h) {
    return any_cast<Prop&>(_property_pool[h._name]);
  }

  template<class Prop>
  const Prop& get_property(const Handle<Prop>& h) const {
    return any_cast<Prop&>(_property_pool[h._name]);
  }

  template<class Prop>
  bool type_property(const string& name, Prop type) {
    try {
      any_cast<Prop>(_property_pool[name]);
      return true;
    } catch (...) {
      return false;
    }
  }
  
  template<class Prop>
  Handle<Prop> get_handle(const string& name, const Prop& p=Prop()) {
    return Handle<Prop>(p, name);
  }
  
};


int
main() {

  PropertyManager pm;
  PropertyManager::Handle<string> h1=pm.request_property("test", string("test"));
  pm.set_property(h1,h1.build_type("test2"));
  cout << pm.get_property(h1) << endl;
  PropertyManager::Handle<int> h2=pm.request_property("test2", 2);

  assert(pm.type_property("test",string()));
  assert(!pm.type_property("test",int()));
  PropertyManager::Handle<string> h3=pm.get_handle<string>("test");
  pm.get_property(h3);
  return 0;
}
